{{
   demo operation of the D6t thermal imaging sensor, 16 pixels 4x4
   Tracy Allen, EME Systems,
   (c) 2014, MIT license, see end of program for terms of use

   If your terminal program doesn't have an emulation, home function
   reduce the size of the terminal window to 5 lines.
   }}


CON
        _clkmode = xtal1 + pll16x     'Standard clock mode * crystal frequency = 10 MHz
        _xinfreq = 5_000_000

  SDA = 6 '14
  SCL = 5 '13

  D6T_ID = $14  ' 8 bit write address
  D6T_CONVERT = $4c

  MAX_I2C_HOLD = 80   ' microseconds

   
OBJ
  uarts       : "fullDuplexSerial4port"   'Debug terminal for communication
  i2c    : "i2c_1bus"

VAR
   byte array[35]
   long pixels[16]
   long errors, times
   long reference

PUB main | ack, idx, jdx, crc
  startUarts
  waitcnt(clkfreq/10+cnt)

  uarts.str(0,string(13,"demo of D6T 8-linear, reading temperatures."))
  if ina[SDA] & ina[SCL] == 0
    uarts.str(0,string(13,"missing pullup"))
    repeat
  if i2c.Init(SCL,SDA) == 1
    uarts.str(0,string(13,"bad i2c"))
    repeat
  if i2c.Hello(D6T_ID) == 1
    uarts.str(0,string(13,"D6T not found"))
    repeat
  uarts.str(0,string(13,"device found"))
  i2c.SetMaxHoldTime(MAX_I2C_HOLD)   ' set hold in microseconds
  errors~
  times~
  repeat
    waitcnt(clkfreq/2+cnt)
    uarts.tx(0,13)
    ack := i2c.readPage(D6T_ID, D6T_CONVERT, @array, 19)
    'repeat idx from 0 to 19
     ' uarts.hex(0,array[idx],2)
      'uarts.tx(0,32)
    'repeat

    crc := Crc8Calc(@array, 18, $c5)
    if crc <> array[18]
      uarts.tx(0,13)
      uarts.chars(0,"?",3)
      quit

'{
    uarts.tx(0,13)   ' to show the data array values
    repeat idx from 0 to 18
        uarts.hex(0,array[idx],2)
        uarts.char(0,32)
    crc := Crc8Calc(@array, 18, $c5)
    uarts.hex(0,crc,2)
'}
    ExtractPixels
    uarts.tx(0,13)
    uarts.decP(0,reference,1)
    repeat idx from 0 to 7
      uarts.decPW(0,pixels[idx],1,7)


PUB ExtractPixels |  idx
  ' reference := ((array[1] & !$00000200 )<< 8) + array[0]
  reference := (array[1]<< 8) + array[0]
  repeat idx from 1 to 8
    pixels[idx-1] := (array[idx*2+1] << 8) + array[idx*2]   - reference

PUB Crc8Calc(dataPointer, dataCount, crc)
' enter with data pointer to data bytes
  'uarts.tx(0,13)
  Repeat dataCount
    crc := Crc8shift(byte[datapointer++] ^ crc)
    {uarts.hex(0,crc,2)
    uarts.char(0,32)}
  return crc

PUB Crc8shift(datum) | temp
  repeat 8
    datum <<= 1
    if datum & $100
        datum ^= $07
  return datum & $ff

PUB StartUarts
  uarts.init
  uarts.addport(0,31,30,-1,-1,0,0,115200)
  uarts.start


{{
┌─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                 │                                                            
├─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation   │ 
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,   │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the        │
│Software is furnished to do so, subject to the following conditions:                                                         │         
│                                                                                                                             │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the         │
│Software.                                                                                                                    │
│                                                                                                                             │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE         │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR        │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,  │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                        │
└─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}  
